options ls=75 ps=65;

data count;
infile 'count.dat';
input patient visit sefreq time trt2@;
output;

/*Poisson GLMM*/

proc nlmixed data=count qpoints=50;
     parms b0=-5 b1=1 b2=0.2 s=1.2 ;
     eta=b0 + b1*trt2 + b2*log(time) + u;
     m=exp(eta);
     model sefreq ~ poisson(m);
     random u~normal(0,s*s) subject= patient;
     run;

/*Random effect type II negative binomial model*/
    
proc nlmixed data=count qpoints=50;
    parms b0=-4 b1=1 b2=1  s=1.5 a=1;
    eta=b0 + b1*trt2 + b2*log(time) + u;
    m=exp(eta);
    p=a*m/(1+a*m);
    ll=lgamma(sefreq+1/a)-lgamma(sefreq+1)-lgamma(1/a);
    ll=ll + 1/a*log(1-p) + sefreq*log(p);
    model sefreq~general(ll);
    random u~normal(0,s*s) subject= patient;
    run;

/*Random effect zero-altered model*/

proc nlmixed data=count qpoints=50;
     parms b0=-5 b1=1 b2=0.2 s1=1.256 b5=-1 ;
     eta1 = b5+ (b0 + b1*trt2 + b2*log(time))+u1;
     expeta1 = exp(eta1);
     eta2 =b0 + b1*trt2 + b2*log(time) + u1;
     expeta2 = exp(eta2);
     if sefreq=0 then ll= -expeta1;
     else ll=log(1-exp(-expeta1))+sefreq*eta2-log(exp(expeta2)-1)-lgamma(sefreq+1);
     model sefreq ~ general(ll);
     contrast 'b5=0'  b5;
     random u1  ~ normal(0,s1*s1) subject=patient;
     run;

/*Random effect hurdle model (1) with log(time) in the first model*/

proc nlmixed data=count qpoints=50;
     bounds -1<= s12 <=1;
     parms b0=-2.874 b1=0.8945  b2=0.02137 s1=1.6469
           b3=-2.844 b4=0.9626 b5=0.5398 s2=0.7060 s12=0.8481;
     eta1 = b0 + b1*trt2 + b2*log(time) + u1;
     expeta1 = exp(eta1);
     eta2 = b3 + b4*trt2 + b5*log(time) + u2;
     expeta2 = exp(eta2);
     if sefreq=0 then ll= -log(1+expeta1);
     else
      ll=eta1-log(1+expeta1)+sefreq*eta2-log(exp(expeta2)-1)-lgamma(sefreq+1);
     model sefreq ~ general(ll);
     contrast 'b2=0'  b2;
     random u1 u2 ~ normal([0,0],[s1*s1,s12*s1*s2,s2*s2]) subject=patient;   
     run;

/*Random effect hurdle model (2) without log(time) in the first model*/

proc nlmixed data=count qpoints=50;
     bounds -1<= s12 <=1;
     parms b0=-2.874 b1=0.8945  s1=1.6469
           b3=-2.844 b4=0.9626 b5=0.5398 s2=0.7060 s12=0.8481;
     eta1 = b0 + b1*trt2  + u1;
     expeta1 = exp(eta1);
     eta2 = b3 + b4*trt2 + b5*log(time) + u2;
     expeta2 = exp(eta2);
     if sefreq=0 then ll= -log(1+expeta1);
     else
      ll=eta1-log(1+expeta1)+sefreq*eta2-log(exp(expeta2)-1)-lgamma(sefreq+1);
     model sefreq ~ general(ll);
     random u1 u2 ~ normal([0,0],[s1*s1,s12*s1*s2,s2*s2]) subject=patient;   
     run;


/*Random effect cumulative logit model with b2*/

proc nlmixed data=count qpoints=50;
      parms i1=3.3 i2=4.62 i3=5.9 i4=6.37 i5=7.55 
            b1=0.9773  b2=0.153 s1=1.749;
      eta1 = i1 - b1*trt2 - b2*log(time) - u;
      eta2 = i2 - b1*trt2 - b2*log(time) - u;
      eta3 = i3 - b1*trt2 - b2*log(time) - u;
      eta4 = i4 - b1*trt2 - b2*log(time) - u;
	  eta5 = i5 - b1*trt2 - b2*log(time) - u;
      if (sefreq=0) then z=1/(1+exp(-eta1));
      else if (sefreq=1) then z=1/(1+exp(-eta2))-1/(1+exp(-eta1));
      else if (sefreq=2) then z=1/(1+exp(-eta3))-1/(1+exp(-eta2));
	  else if (sefreq=3) then z=1/(1+exp(-eta4))-1/(1+exp(-eta3));
      else if (sefreq=4) then z=1/(1+exp(-eta5))-1/(1+exp(-eta4));
      else z=1-1/(1+exp(-eta5));
      if (z>1e-8) then ll=log(z);
      else ll=-1e100;
      model sefreq ~ general(ll);
      random u ~ normal(0,s1*s1) subject=patient;
      run;

/*Random effect cumulative logit model without b2*/

proc nlmixed data=count qpoints=50;
      parms i1=3.3 i2=4.62 i3=5.9 i4=6.37 i5=7.55 
            b1=0.9773 s1=1.749;
      eta1 = i1 - b1*trt2  - u;
      eta2 = i2 - b1*trt2  - u;
      eta3 = i3 - b1*trt2  - u;
      eta4 = i4 - b1*trt2  - u;
	  eta5 = i5 - b1*trt2  - u;
      if (sefreq=0) then z=1/(1+exp(-eta1));
      else if (sefreq=1) then z=1/(1+exp(-eta2))-1/(1+exp(-eta1));
      else if (sefreq=2) then z=1/(1+exp(-eta3))-1/(1+exp(-eta2));
	  else if (sefreq=3) then z=1/(1+exp(-eta4))-1/(1+exp(-eta3));
      else if (sefreq=4) then z=1/(1+exp(-eta5))-1/(1+exp(-eta4));
      else z=1-1/(1+exp(-eta5));
      if (z>1e-8) then ll=log(z);
      else ll=-1e100;
      model sefreq ~ general(ll);
      random u ~ normal(0,s1*s1) subject=patient;
      run;
